@@ -919,7 +919,7 @@ func TestTruncateTreeNotArray(t *testing.T) {
 func TestTruncateTreeArrayOutOfBounds(t *testing.T) {
 	tree := TreeBranch{
 		TreeItem{
-			Key:   "foo",
+			Key: "foo",
 			Value: []interface{}{
 				"one",
 				"two",
@@ -1023,10 +1023,33 @@ func TestSetNewKey(t *testing.T) {
 			},
 		},
 	}
-	set := branch.Set([]interface{}{"foo", "bar", "foo"}, "hello")
+	set, changed := branch.Set([]interface{}{"foo", "bar", "foo"}, "hello")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, "hello", set[0].Value.(TreeBranch)[0].Value.(TreeBranch)[1].Value)
 }
 
+func TestSetNewKeyUnchanged(t *testing.T) {
+	branch := TreeBranch{
+		TreeItem{
+			Key: "foo",
+			Value: TreeBranch{
+				TreeItem{
+					Key: "bar",
+					Value: TreeBranch{
+						TreeItem{
+							Key:   "baz",
+							Value: "foobar",
+						},
+					},
+				},
+			},
+		},
+	}
+	set, changed := branch.Set([]interface{}{"foo", "bar", "baz"}, "foobar")
+	assert.Equal(t, false, changed)
+	assert.Equal(t, "foobar", set[0].Value.(TreeBranch)[0].Value.(TreeBranch)[0].Value)
+}
+
 func TestSetNewBranch(t *testing.T) {
 	branch := TreeBranch{
 		TreeItem{
@@ -1034,7 +1057,8 @@ func TestSetNewBranch(t *testing.T) {
 			Value: "value",
 		},
 	}
-	set := branch.Set([]interface{}{"foo", "bar", "baz"}, "hello")
+	set, changed := branch.Set([]interface{}{"foo", "bar", "baz"}, "hello")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
 			Key:   "key",
@@ -1067,7 +1091,8 @@ func TestSetArrayDeepNew(t *testing.T) {
 			},
 		},
 	}
-	set := branch.Set([]interface{}{"foo", 2, "bar"}, "hello")
+	set, changed := branch.Set([]interface{}{"foo", 2, "bar"}, "hello")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, "hello", set[0].Value.([]interface{})[2].(TreeBranch)[0].Value)
 }
 
@@ -1078,13 +1103,15 @@ func TestSetNewKeyDeep(t *testing.T) {
 			Value: "bar",
 		},
 	}
-	set := branch.Set([]interface{}{"foo", "bar", "baz"}, "hello")
+	set, changed := branch.Set([]interface{}{"foo", "bar", "baz"}, "hello")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, "hello", set[0].Value.(TreeBranch)[0].Value.(TreeBranch)[0].Value)
 }
 
 func TestSetNewKeyOnEmptyBranch(t *testing.T) {
 	branch := TreeBranch{}
-	set := branch.Set([]interface{}{"foo", "bar", "baz"}, "hello")
+	set, changed := branch.Set([]interface{}{"foo", "bar", "baz"}, "hello")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, "hello", set[0].Value.(TreeBranch)[0].Value.(TreeBranch)[0].Value)
 }
 
@@ -1099,13 +1126,15 @@ func TestSetArray(t *testing.T) {
 			},
 		},
 	}
-	set := branch.Set([]interface{}{"foo", 0}, "uno")
+	set, changed := branch.Set([]interface{}{"foo", 0}, "uno")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, "uno", set[0].Value.([]interface{})[0])
 }
 
 func TestSetArrayNew(t *testing.T) {
 	branch := TreeBranch{}
-	set := branch.Set([]interface{}{"foo", 0, 0}, "uno")
+	set, changed := branch.Set([]interface{}{"foo", 0, 0}, "uno")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, "uno", set[0].Value.([]interface{})[0].([]interface{})[0])
 }
 
@@ -1116,7 +1145,8 @@ func TestSetExisting(t *testing.T) {
 			Value: "foobar",
 		},
 	}
-	set := branch.Set([]interface{}{"foo"}, "bar")
+	set, changed := branch.Set([]interface{}{"foo"}, "bar")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, "bar", set[0].Value)
 }
 
@@ -1127,7 +1157,8 @@ func TestSetArrayLeafNewItem(t *testing.T) {
 			Value: []interface{}{},
 		},
 	}
-	set := branch.Set([]interface{}{"array", 2}, "hello")
+	set, changed := branch.Set([]interface{}{"array", 2}, "hello")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
 			Key: "array",
@@ -1147,7 +1178,8 @@ func TestSetArrayNonLeaf(t *testing.T) {
 			},
 		},
 	}
-	set := branch.Set([]interface{}{"array", 0, "hello"}, "hello")
+	set, changed := branch.Set([]interface{}{"array", 0, "hello"}, "hello")
+	assert.Equal(t, true, changed)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
 			Key: "array",
@@ -1166,11 +1198,11 @@ func TestSetArrayNonLeaf(t *testing.T) {
 func TestUnsetKeyRootLeaf(t *testing.T) {
 	branch := TreeBranch{
 		TreeItem{
-			Key: "foo",
+			Key:   "foo",
 			Value: "foo",
 		},
 		TreeItem{
-			Key: "foofoo",
+			Key:   "foofoo",
 			Value: "foofoo",
 		},
 	}
@@ -1178,7 +1210,7 @@ func TestUnsetKeyRootLeaf(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
-			Key: "foo",
+			Key:   "foo",
 			Value: "foo",
 		},
 	}, unset)
@@ -1190,11 +1222,11 @@ func TestUnsetKeyBranchLeaf(t *testing.T) {
 			Key: "foo",
 			Value: TreeBranch{
 				TreeItem{
-					Key: "bar",
+					Key:   "bar",
 					Value: "bar",
 				},
 				TreeItem{
-					Key: "barbar",
+					Key:   "barbar",
 					Value: "barbar",
 				},
 			},
@@ -1207,7 +1239,7 @@ func TestUnsetKeyBranchLeaf(t *testing.T) {
 			Key: "foo",
 			Value: TreeBranch{
 				TreeItem{
-					Key: "bar",
+					Key:   "bar",
 					Value: "bar",
 				},
 			},
@@ -1218,14 +1250,14 @@ func TestUnsetKeyBranchLeaf(t *testing.T) {
 func TestUnsetKeyBranch(t *testing.T) {
 	branch := TreeBranch{
 		TreeItem{
-			Key: "foo",
+			Key:   "foo",
 			Value: "foo",
 		},
 		TreeItem{
 			Key: "foofoo",
 			Value: TreeBranch{
 				TreeItem{
-					Key: "bar",
+					Key:   "bar",
 					Value: "bar",
 				},
 			},
@@ -1235,7 +1267,7 @@ func TestUnsetKeyBranch(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
-			Key: "foo",
+			Key:   "foo",
 			Value: "foo",
 		},
 	}, unset)
@@ -1244,14 +1276,13 @@ func TestUnsetKeyBranch(t *testing.T) {
 func TestUnsetKeyRootLastLeaf(t *testing.T) {
 	branch := TreeBranch{
 		TreeItem{
-			Key: "foo",
+			Key:   "foo",
 			Value: "foo",
 		},
 	}
 	unset, err := branch.Unset([]interface{}{"foo"})
 	assert.NoError(t, err)
-	assert.Equal(t, TreeBranch{
-	}, unset)
+	assert.Equal(t, TreeBranch{}, unset)
 }
 
 func TestUnsetKeyBranchLastLeaf(t *testing.T) {
@@ -1260,7 +1291,7 @@ func TestUnsetKeyBranchLastLeaf(t *testing.T) {
 			Key: "foo",
 			Value: TreeBranch{
 				TreeItem{
-					Key: "bar",
+					Key:   "bar",
 					Value: "bar",
 				},
 			},
@@ -1270,9 +1301,8 @@ func TestUnsetKeyBranchLastLeaf(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
-			Key: "foo",
-			Value: TreeBranch{
-			},
+			Key:   "foo",
+			Value: TreeBranch{},
 		},
 	}, unset)
 }
@@ -1287,7 +1317,7 @@ func TestUnsetKeyArray(t *testing.T) {
 					Value: []interface{}{
 						TreeBranch{
 							TreeItem{
-								Key: "baz",
+								Key:   "baz",
 								Value: "baz",
 							},
 						},
@@ -1300,9 +1330,8 @@ func TestUnsetKeyArray(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
-			Key: "foo",
-			Value: TreeBranch{
-			},
+			Key:   "foo",
+			Value: TreeBranch{},
 		},
 	}, unset)
 }
@@ -1314,13 +1343,13 @@ func TestUnsetArrayItem(t *testing.T) {
 			Value: []interface{}{
 				TreeBranch{
 					TreeItem{
-						Key: "bar",
+						Key:   "bar",
 						Value: "bar",
 					},
 				},
 				TreeBranch{
 					TreeItem{
-						Key: "barbar",
+						Key:   "barbar",
 						Value: "barbar",
 					},
 				},
@@ -1335,7 +1364,7 @@ func TestUnsetArrayItem(t *testing.T) {
 			Value: []interface{}{
 				TreeBranch{
 					TreeItem{
-						Key: "bar",
+						Key:   "bar",
 						Value: "bar",
 					},
 				},
@@ -1351,11 +1380,11 @@ func TestUnsetKeyInArrayItem(t *testing.T) {
 			Value: []interface{}{
 				TreeBranch{
 					TreeItem{
-						Key: "bar",
+						Key:   "bar",
 						Value: "bar",
 					},
 					TreeItem{
-						Key: "barbar",
+						Key:   "barbar",
 						Value: "barbar",
 					},
 				},
@@ -1370,7 +1399,7 @@ func TestUnsetKeyInArrayItem(t *testing.T) {
 			Value: []interface{}{
 				TreeBranch{
 					TreeItem{
-						Key: "bar",
+						Key:   "bar",
 						Value: "bar",
 					},
 				},
@@ -1386,7 +1415,7 @@ func TestUnsetArrayLastItem(t *testing.T) {
 			Value: []interface{}{
 				TreeBranch{
 					TreeItem{
-						Key: "bar",
+						Key:   "bar",
 						Value: "bar",
 					},
 				},
@@ -1397,9 +1426,8 @@ func TestUnsetArrayLastItem(t *testing.T) {
 	assert.NoError(t, err)
 	assert.Equal(t, TreeBranch{
 		TreeItem{
-			Key: "foo",
-			Value: []interface{}{
-			},
+			Key:   "foo",
+			Value: []interface{}{},
 		},
 	}, unset)
 }
@@ -1410,7 +1438,7 @@ func TestUnsetKeyNotFound(t *testing.T) {
 			Key: "foo",
 			Value: TreeBranch{
 				TreeItem{
-					Key: "bar",
+					Key:   "bar",
 					Value: "bar",
 				},
 			},
@@ -1429,7 +1457,7 @@ func TestUnsetKeyInArrayNotFound(t *testing.T) {
 			Value: []interface{}{
 				TreeBranch{
 					TreeItem{
-						Key: "bar",
+						Key:   "bar",
 						Value: "bar",
 					},
 				},
@@ -1448,7 +1476,7 @@ func TestUnsetArrayItemOutOfBounds(t *testing.T) {
 			Value: []interface{}{
 				TreeBranch{
 					TreeItem{
-						Key: "bar",
+						Key:   "bar",
 						Value: "bar",
 					},
 				},
@@ -1463,7 +1491,7 @@ func TestUnsetArrayItemOutOfBounds(t *testing.T) {
 func TestUnsetKeyNotABranch(t *testing.T) {
 	branch := TreeBranch{
 		TreeItem{
-			Key: "foo",
+			Key:   "foo",
 			Value: 99,
 		},
 	}
