@@ -24,7 +24,8 @@ func NewStore(c *config.INIStoreConfig) *Store {
 }
 
 func (store Store) encodeTree(branches sops.TreeBranches) ([]byte, error) {
-	iniFile := ini.Empty()
+	iniFile := ini.Empty(ini.LoadOptions{AllowNonUniqueSections: true})
+	iniFile.DeleteSection(ini.DefaultSection)
 	for _, branch := range branches {
 		for _, item := range branch {
 			if _, ok := item.Key.(sops.Comment); ok {
@@ -95,7 +96,7 @@ func (store Store) iniFromTreeBranches(branches sops.TreeBranches) ([]byte, erro
 }
 
 func (store Store) treeBranchesFromIni(in []byte) (sops.TreeBranches, error) {
-	iniFile, err := ini.Load(in)
+	iniFile, err := ini.LoadSources(ini.LoadOptions{AllowNonUniqueSections: true}, in)
 	if err != nil {
 		return nil, err
 	}
@@ -143,7 +144,7 @@ func (store Store) treeItemFromSection(section *ini.Section) (sops.TreeItem, err
 
 // LoadEncryptedFile loads encrypted INI file's bytes onto a sops.Tree runtime object
 func (store *Store) LoadEncryptedFile(in []byte) (sops.Tree, error) {
-	iniFileOuter, err := ini.Load(in)
+	iniFileOuter, err := ini.LoadSources(ini.LoadOptions{AllowNonUniqueSections: true}, in)
 	if err != nil {
 		return sops.Tree{}, err
 	}
